@@ -9,9 +9,12 @@ import dateutil.parser as parser
 import string
 import random
 import shortuuid
+import hashlib
+from json_logic import jsonLogic
 DT_FMT_HMSf = '%H%M%S%f'
 
 
+
 class HttpMethodEnum(Enum):
     POST = 'POST'
     PUT = 'PUT'
@@ -156,8 +159,8 @@ def buildquery(json_object):
         if json_object.get("operator") != 'BETWEEN':
             return json_object.get("field") + " " + json_object.get("operator") + " '" + json_object.get("value") + "' "
         else:
-            return json_object.get("field") + " " + json_object.get("operator") + " '" + json_object.get("value") + "' " + 'AND' + " '" + json_object.get("value2")+ "' "
-
+            return json_object.get("field") + " " + json_object.get("operator") + " '" + json_object.get("value") + "' " + 'AND' + " '" + json_object.get("value2") + "' "
+        
     # else it is an "condition+filters" JSON - Object
     else:
         i = 0
@@ -176,3 +179,117 @@ def buildquery(json_object):
             i = i + 1
         # result += ")"
         return result
+
+    
+def get_session_name(user_access_key, vtiger_url, vtiger_username):
+    # get challenge
+    headers = {'content-type': 'application/json'}    
+    challenge_url = '{vtiger_url}/webservice.php?operation=getchallenge&username={vtiger_username}'.format(vtiger_url=vtiger_url, vtiger_username=vtiger_username)
+    response, status = invoke_http_request(challenge_url, 'GET', headers=headers)
+
+    if status == 200 and response.get('result', ''):
+        token = response.get('result').get('token')
+
+        # get md5 encoded access key
+
+        access_key = token + user_access_key
+        result = hashlib.md5(access_key.encode())
+        access_key_encoded = result.hexdigest()
+
+        # login
+        login_url = "{vtiger_url}/webservice.php".format(vtiger_url=vtiger_url)
+        payload = 'operation=login&username={vtiger_username}&accessKey={access_key}'.format(access_key=access_key_encoded, vtiger_username=vtiger_username)
+        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
+
+        response, status = invoke_http_request(login_url, 'POST', headers, payload)
+
+        if status == 200 and response.get('result', ''):
+            return response.get('result').get('sessionName')
+
+        
+def trigger_workflow(workflow, event_type, data, service_url):
+    """ call API to trigger any workflow
+        Required params: 1. workflow 2. event_type 3. data"""
+
+    headers = {'content_type': 'application/json'}
+    data['eventtype'] = event_type
+
+    endpoint = '{service_url}/api/v1/trigger_external_workflow'.format(service_url=service_url)
+
+    if data:
+        payload = {
+            "workflow": workflow,
+            "data": data
+        }
+
+    response, status = invoke_http_request(endpoint, 'POST', headers, json_data=payload, timeout=61)
+    print("response", response, "status", status)
+
+    
+def json_logic_replace_data(rule, data, string_data):
+    replace_data = jsonLogic(rule, data)
+    it = iter(replace_data)
+    res_dct = dict(zip(it, it))
+
+    for key, val in res_dct.items():
+        string_data = string_data.replace(key, str(val))
+
+    return string_data
+
+
+def run_external_workflow(conf, external_workflow_config, vtiger_access):
+    """ this function will get check if conditions are satisfied for triggering external workflow or not.
+        input:  1.conf : conf object
+                2.external_workflow_config : config dictionary containing workflow, event_type, search_object
+        1. get configs and create query.
+        2. replace data using json logic.
+        3. call trigger workflow function if conditions are satisfied.
+        """
+
+    data = conf.get('data', '')
+    workflow = external_workflow_config.get('workflow', '')
+    event_type = external_workflow_config.get('event_type', '')
+
+    search_object = external_workflow_config.get('search_object', '')
+
+    if search_object and search_object.get('condition_object', ''):
+        rule = search_object.get('rule', '')
+
+        # prepare query
+        condition = buildquery(search_object.get('condition_object'))
+        module = search_object.get('search_module').get('name')
+        limit = str(search_object.get('fetch_record'))
+
+        order_by = search_object.get('sort').get('column') + " " + search_object.get('sort').get('type', "")
+
+        query = 'SELECT * FROM {module} WHERE {condition} order by {order_by} LIMIT {limit};'.format(
+            module=module,
+            condition=condition,
+            order_by=order_by,
+            limit=limit)
+
+        # replace variable name with data using JSON_LOGIC.
+
+        if rule:
+            query = json_logic_replace_data(rule, conf, query)
+
+        if vtiger_access.get('user_access_key', ''):
+            session_name = get_session_name(vtiger_access.get('user_access_key'), vtiger_access.get('vtiger_url'), vtiger_access.get('vtiger_username'))
+            if not session_name:
+                print("unable to get session id from dev server. Please try again")
+                return None
+
+            url = '{vtiger_url}/webservice.php?operation=query&sessionName={sessionName}&query={query}'.format(
+                sessionName=session_name, query=query, vtiger_url=vtiger_access.get('vtiger_url'))
+            headers = {'content-type': 'application/json'}
+            request_type = 'GET'
+            response, status = invoke_http_request(url, request_type, headers)
+    
+            if response:
+                # trigger only if condition is satisfied
+                if response.get('result') and workflow and event_type:
+                    trigger_workflow(workflow, event_type, response.get('result')[0], vtiger_access.get('service_url'))
+    elif workflow and event_type and data:
+        # trigger without condition
+        trigger_workflow(workflow, event_type, data, vtiger_access.get('service_url'))
+  
